@@ -114,7 +114,7 @@
     (write-string
      (etypecase name
        (string name)
-       (symbol (symbol-name name) database))
+       (symbol (symbol-name name)))
      *sql-stream*))
   t)
 
@@ -571,7 +571,8 @@ uninclusive, and the args from that keyword to the end."
         (write-string "ON " *sql-stream*)
         (output-sql distinct database)
         (write-char #\Space *sql-stream*)))
-    (output-sql (apply #'vector selections) database)
+    (let ((*in-subselect* t))
+      (output-sql (apply #'vector selections) database))
     (when from
       (write-string " FROM " *sql-stream*)
       (flet ((ident-table-equal (a b)
